#!/usr/bin/perl -w

$| = 1;

# ddrcopyincremnt.pl
#
# The situation is:  We did a ddrescue copy of /dev/olddrive to temp.ima
# (or to a new drive), generating drive.log, which we saved to olddrive.log,
# but some sectors were not succesfully recovered.  Then, we copied temp.ima
# to a temporary drive.  While copying it, or afterward, we did "freezer
# recovery" or SpinRite recover or just let ddrescue run a while longer
# copying from the old drive to temp.ima, and it rescued a few more sectors.
# Now we want to copy just the additional sectors to the temporary drive.
#
# Here's the strategy.  The additional sectors are those which are
# good ("+") in drive.log, but bad ("-" or "/") in olddrive.log.
# We need a logfile which shows only those additional sectors as
# bad, so that ddrescue will copy only those additional sectors.
# I.e., we want:
#
#    olddrive.log   newdrive.log   temp.log
#         -              +            -
#         +              +            +
#         -              -            +
#         +              -       (impossible)
#
# I.e., if a sector is "-" in newdrive.log or if it is "+" in
# olddrive.log then it will be "+" in temp.log.
#
# I.e.,
# temp.log = oldrive.log .OR. (.NOT. newdrive.log)
#
# Usage:
#
#  perl ddrincrementalcopy.pl temp.log olddrive.log drive.log
#    (that creates temp.log from the two ddrescue logfiles)
#  ddrescue -r 1 drive.ima /dev/tempdrive temp.log
#    (that updates /dev/tempdrive from the updated drive.ima)
#

$whatdirectory = '';
if ($0 =~ /^(.*[\/\\])([^\/\\]+)$/) {
   $whatdirectory = $1;
   $whatprogramname = $2;
   # print "dbg: directory='$whatdirectory'\n";
}

if (2 != $#ARGV) {
   print "$whatprogramname -- update a disk with additional recovered sectors\n" .
         "\n" .
         "Usage:\n" .
         "  perl -w $0 temp.log olddrive.log drive.log\n" .
         "     (that creates temp.log from the two ddrescue logfiles)\n" .
         "  ddrescue -r 1 drive.ima /dev/tempdrive temp.log\n" .
         "     (that updates /dev/tempdrive from the updated drive.ima)\n" .
         "\n" .
         "The scenario as follows:  First, you do a ddrescue recovery of a bad disk\n" .
         "drive into drive.ima (or another drive).  Then you save a copy of ddrescue's\n" .
         "drive.log file to olddrive.log.  Then (as ddrescue continues to try to recover\n" .
         "additional sectors) you use dd or a second instance of ddrescue to copy\n" .
         "drive.ima to a new drive (perhaps a scratch drive) which we'll call\n" .
         "/dev/tempdrive.  After a few hours, the copy to /dev/tempdrive is finally\n" .
         "done, but by then the original copy of ddrescue has managed to recover some\n" .
         "additional sectors from the bad drive into drive.ima.  Now we want to quickly\n" .
         "update /dev/tempdrive with those newly recovered sectors.  That's what\n" .
         "$whatprogramname is for.\n";
   exit(1);
}

($templog, $oldlog, $newlog) = @ARGV;


$cmd = "perl -w " . $whatdirectory . "ddrlognot.pl $newlog $templog";
print "-- about to run '$cmd' --\n";
print `$cmd`;
print "-- back from running '$cmd' --\n";
print "\n";
$cmd = "perl -w " . $whatdirectory . "ddrlogor.pl -o $templog $oldlog $templog";
print "-- about to run '$cmd' --\n";
print `$cmd`;
print "--back from running '$cmd' --\n";
print "____________________________________________\n";
print "Output file is '$templog'.  Use it like this\n";
print "  ddrescue -r 1 -C NewDriveOrImage TempDriveToUpdate $templog\n";

__END__
